﻿using System.Windows.Input;

namespace Zachowania
{
    internal class PrzyciskZLicznikiem : Behavior<Button>
    {
        public int Licznik { get; set; } = 0;

        protected override void OnAttachedTo(Button button)
        {
            base.OnAttachedTo(button);
            if(button != null)
            {
                button.Clicked += button_Clicked;
                button.Text = "Licznik: " + Licznik.ToString();
                button.BackgroundColor = Colors.Red;
            }
        }

        protected override void OnDetachingFrom(Button button)
        {
            base.OnDetachingFrom(button);
            button.Clicked -= button_Clicked;
        }

        private void button_Clicked(object? sender, EventArgs e)
        {
            Licznik++;
            Button? button = sender as Button;
            if (button != null)
            {
                button.Text = "Licznik: " + Licznik.ToString();
                button.BackgroundColor = Colors.Lime;
            }
        }
    }

    internal class PrzyciskZmieniającyTło : Behavior<VisualElement>
    {
        public VisualElement VisualElement { get; private set; } = null;

        protected override void OnAttachedTo(VisualElement visualElement)
        {
            base.OnAttachedTo(visualElement);
            this.VisualElement = visualElement;
        }

        //własność wiązania/wiązalną = własność zależności
        public static readonly BindableProperty PrzyciskProperty =
            BindableProperty.Create(
                nameof(Przycisk),
                typeof(Button),
                typeof(PrzyciskZmieniającyTło),
                null,
                BindingMode.OneWay,
                null,
                przyciskZmieniony);

        public Button Przycisk
        {
            get { return (Button)GetValue(PrzyciskProperty); }
            set { SetValue(PrzyciskProperty, value);}
        }

        public Color Kolor { get; set; } = Colors.Yellow;

        private static void przyciskZmieniony(BindableObject b, object oldValue, object newValue)
        {
            EventHandler button_Click =
                (object sender, EventArgs e) =>
                {
                    PrzyciskZmieniającyTło _b = b as PrzyciskZmieniającyTło;
                    if (_b.VisualElement != null)
                        _b.VisualElement.Background = _b.Kolor;
                    //wykonanie polecenia
                    ICommand command = _b.Command;
                    object commandParameter = _b.CommandParameter;
                    if (command != null && command.CanExecute(commandParameter)) 
                        command.Execute(commandParameter);
                };
            if(oldValue != null) (oldValue as Button).Clicked -= button_Click;
            if(newValue != null) (newValue as Button).Clicked += button_Click;
        }

        #region Polecenie
        public static readonly BindableProperty CommandProperty =
            BindableProperty.Create(
                nameof(Command),
                typeof(ICommand),
                typeof(PrzyciskZmieniającyTło),
                null);

        public ICommand Command
        {
            get => (ICommand)GetValue(CommandProperty); 
            set => SetValue(CommandProperty, value);
        }

        public static readonly BindableProperty CommandParameterProperty =
            BindableProperty.Create(
                nameof(CommandParameter),
                typeof(object),
                typeof(PrzyciskZmieniającyTło),
                null);

        public object CommandParameter
        {
            get => GetValue(CommandParameterProperty);
            set => SetValue(CommandParameterProperty, value);
        }
        #endregion
    }

    public static class ZmianaTłaBehavior
    {
        public static Color GetKolor(BindableObject d)
        {
            return (Color)d.GetValue(KolorProperty);
        }

        public static void SetKolor(BindableObject d, Color kolor)
        {
            d.SetValue(KolorProperty, kolor);
        }

        public static readonly BindableProperty KolorProperty =
            BindableProperty.CreateAttached(
                "Kolor",
                typeof(Color),
                typeof(ZmianaTłaBehavior),
                null,
                BindingMode.OneWay,
                null,
                kolorZmieniony);

        private static void kolorZmieniony(BindableObject d, object oldValue, object newValue)
        {
            Color kolor = (Color)newValue;
            Brush brush = new SolidColorBrush(kolor);
            if(d is VisualElement) 
                (d as VisualElement).Background = brush;
        }
    }
}
